package org.crosswire.android.bishop;

import java.util.Locale;

import android.app.Activity;
import android.app.ProgressDialog;
import android.content.Intent;
import android.os.Bundle;
import android.os.Handler;
import android.os.Message;
import android.speech.tts.TextToSpeech;
import android.view.KeyEvent;
import android.view.Menu;
import android.view.MenuItem;
import android.webkit.WebView;
import android.widget.LinearLayout;
import android.widget.TextView;
import android.widget.LinearLayout.LayoutParams;

import org.crosswire.android.sword.SWMgr;
import org.crosswire.android.sword.SWModule;

public class Bishop extends Activity {
	About about = null;
	GotoDialog gotoDialog = null;
    ProgressDialog searchProgress;
    SWMgr mgr;
	LinearLayout mainLayout;
	WebView  wv;
	String currentMod = "";
	String currentKey = "";
    /** Called when the activity is first created. */
    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        this.setTitle("The SWORD Project");
        
        mgr = new SWMgr();
        
		mainLayout = new LinearLayout(this);
		mainLayout.setOrientation(LinearLayout.VERTICAL);

		wv = new WebView(this) {
			@Override
			public boolean onKeyDown(int keyCode, KeyEvent event) {
				if (keyCode == KeyEvent.KEYCODE_DPAD_LEFT) {
					SWModule module = mgr.getModuleByName(currentMod);
					module.setKeyText(currentKey);
					module.setKeyText("-chapter");
					loadContext(currentMod, module.getKeyText());
					return true;
				}
				else if (keyCode == KeyEvent.KEYCODE_DPAD_RIGHT) {
					SWModule module = mgr.getModuleByName(currentMod);
					module.setKeyText(currentKey);
					module.setKeyText("+chapter");
					loadContext(currentMod, module.getKeyText());
					return true;
				}
				else if (keyCode == KeyEvent.KEYCODE_DPAD_CENTER) {
					gotoDialog.askUser(mgr.getModuleByName(currentMod));
					return true;
				}
				return super.onKeyDown(keyCode, event);
			}
		};
		wv.setPadding(10, 10, 10, 10);
		mainLayout.addView(wv);

		setContentView(mainLayout, new LinearLayout.LayoutParams(LayoutParams.FILL_PARENT, LayoutParams.FILL_PARENT));

        SWMgr.ModInfo mods[] = mgr.getModInfoList();
        for (SWMgr.ModInfo mi : mods) {
        	if (mi.category.startsWith("Biblical")) {
        		loadContext(mi.name, "jas.1.19");
        		break;
        	}
        }
        
		about = new About(this);
		about.setOwnerActivity(this);
		gotoDialog = new GotoDialog(this, new GotoDialog.GotoCompleteAction() {
			public void doAction() {
				loadContext(gotoDialog.module.getName(), gotoDialog.module.getKeyText());
			}
		});
		gotoDialog.setOwnerActivity(this);
	    searchProgress = new ProgressDialog(this);
	    searchProgress.setTitle("Search");
	    
	    Intent checkIntent = new Intent();
	    checkIntent.setAction(TextToSpeech.Engine.ACTION_CHECK_TTS_DATA);
	    startActivityForResult(checkIntent, MY_DATA_CHECK_CODE);

    }
    
	private static final int SEARCH_FINISHED = Menu.FIRST;
	private static final int SEARCH_REPORT_PROGRESS = Menu.FIRST + 1;
	private static final int GOTO_ID = Menu.FIRST + 2;
	private static final int COMMENTARY_ID = Menu.FIRST + 3;
	private static final int SEARCH_ID = Menu.FIRST + 4;
	private static final int ABOUT_ID = Menu.FIRST + 5;
	private static final int MY_DATA_CHECK_CODE = Menu.FIRST + 6;
	private static final int SPEAK_ID = Menu.FIRST + 7;
	private static final int INSTALLER_ID = Menu.FIRST + 8;
	
	Handler myHandler = new Handler() {

		public void handleMessage(Message msg) {
			switch (msg.what) {
			case SEARCH_REPORT_PROGRESS:
				searchProgress.setProgress(msg.getData().getInt("progress"));
				break;
			case SEARCH_FINISHED:
				wv.loadData(msg.getData().getString("message"), "text/html", "utf-8");
				searchProgress.hide();
				break;
			default:
				super.handleMessage(msg);
			}
		}
	};
	
	MenuItem commentaryMenuItem;
	public boolean onCreateOptionsMenu(Menu menu) {
		super.onCreateOptionsMenu(menu);

		menu.add(0, GOTO_ID, 0, "Goto").setShortcut('1', 'g');
		commentaryMenuItem = 
		menu.add(2, COMMENTARY_ID, 0, "Commentary").setShortcut('2', 'c');
		menu.add(0, SEARCH_ID, 0, "Search").setShortcut('3', 's');
		menu.add(0, SPEAK_ID, 0, "Speak").setShortcut('4', 't');
		menu.add(0, INSTALLER_ID, 0, "Installer").setShortcut('5', 'i');
		menu.add(0, ABOUT_ID, 0, "About").setShortcut('7', 'a');
		menu.setGroupEnabled(2, false);
  
		return true;
	}
	
	
	@Override
	public boolean onOptionsItemSelected(MenuItem item) {
		switch (item.getItemId()) {
		case GOTO_ID:
			gotoDialog.askUser(mgr.getModuleByName(currentMod));
			return true;
		case SPEAK_ID:
			speakContext();
			return true;
		case COMMENTARY_ID:
			return true;
		case SEARCH_ID:
			startSearch(currentMod, "God love world");
			return true;
		case INSTALLER_ID:
			startActivity(new Intent(this, Installer.class));
		case ABOUT_ID:
			about.show();
			return true;
		}
		return super.onOptionsItemSelected(item);
	}
	
    void loadContext(String mod, String key) {
		SWModule module = mgr.getModuleByName(mod);
		module.setKeyText(key);
		currentMod = mod;
		currentKey = key;
		String keyComponents[] = module.getKeyChildren();
		String bookName = keyComponents[6];
		String chapter  = keyComponents[2];
		String verse    = keyComponents[3];
		String str = "<center><h2>" + bookName + "</h2></center>";
		str += "<center><h3>Chapter " + chapter + "</h3></center>";
		for (module.setKeyText(bookName + " " + chapter + ":1"); module.error() == 0; module.next()) {
			keyComponents = module.getKeyChildren();
			String iBookName = keyComponents[6];
			String iChapter  = keyComponents[2];
			String iVerse    = keyComponents[3];
			if (!iBookName.equals(bookName) || !iChapter.equals(chapter)) {
				break;
			}
			if (iVerse.equals(verse)) {
				str += "<span style=\"background-color:#00ee99;\">";
			}
			str += "<br/><super>" + iVerse + "</super> " + module.getRenderText();
			if (iVerse.equals(verse)) {
				str += "</span>";
			}
	    }
		
		wv.loadData(str, "text/html", "utf-8");
		module.setKeyText(key);
    }
    
	void startSearch(String mod, String searchTerm) {
		final SWModule module = mgr.getModuleByName(mod);
		final String expression = searchTerm;
	    searchProgress.setMessage(expression);
	    //, true, false);
	    searchProgress.setMax(100);
	    searchProgress.setProgressStyle(ProgressDialog.STYLE_HORIZONTAL);
	    searchProgress.show();
		new Thread(new Runnable() {
			public void run() {
				String moreOutput = "";
		        SWModule.SearchHit results [] = module.search(expression, SWModule.SEARCHTYPE_MULTIWORD, 0, "", new SWModule.SearchProgressReporter() {
					public void progressReport(int percent) {
						Message m = myHandler.obtainMessage(SEARCH_REPORT_PROGRESS);
						Bundle b = new Bundle();
						b.putInt("progress", percent);
						m.setData(b);
						myHandler.sendMessage(m);
					}
				});
		        moreOutput += "Search for: " + expression + " (" + results.length + ")<br/><br/>";
		        for (SWModule.SearchHit h : results) {
		        	moreOutput += h.key + "<br/>";
		        }
				Message m = myHandler.obtainMessage(SEARCH_FINISHED);
				Bundle b = new Bundle();
				b.putString("message", moreOutput);
					m.setData(b);
					myHandler.sendMessage(m);
	    		}
			}).start();
	}
		        

    void speakContext() {
		SWModule module = mgr.getModuleByName(currentMod);
		module.setKeyText(currentKey);
		String keyComponents[] = module.getKeyChildren();
		String bookName = keyComponents[6];
		String chapter  = keyComponents[2];
		String verse    = keyComponents[3];
		mTts.speak(bookName, TextToSpeech.QUEUE_FLUSH, null);
		mTts.speak("Chapter " + chapter, TextToSpeech.QUEUE_ADD, null);
		for (module.setKeyText(bookName + " " + chapter + ":1"); module.error() == 0; module.next()) {
			keyComponents = module.getKeyChildren();
			String iBookName = keyComponents[6];
			String iChapter  = keyComponents[2];
			String iVerse    = keyComponents[3];
			if (!iBookName.equals(bookName) || !iChapter.equals(chapter)) {
				break;
			}
			mTts.speak(iVerse + ", " + module.getStripText(), TextToSpeech.QUEUE_ADD, null);
	    }
		module.setKeyText(currentKey);
    }


	private TextToSpeech mTts;
	protected void onActivityResult(
	        int requestCode, int resultCode, Intent data) {
	    if (requestCode == MY_DATA_CHECK_CODE) {
	        if (resultCode == TextToSpeech.Engine.CHECK_VOICE_DATA_PASS) {
	            // success, create the TTS instance
	            mTts = new TextToSpeech(this, null);
	            mTts.setLanguage(Locale.US);
	        } else {
	            // missing data, install it
	            Intent installIntent = new Intent();
	            installIntent.setAction(
	                TextToSpeech.Engine.ACTION_INSTALL_TTS_DATA);
	            startActivity(installIntent);
	        }
	    }
	}

}
